package br.ufrj.cos.lens.odyssey.tracer;

import br.ufrj.cos.lens.odyssey.persistence.xml.*;

import java.io.PrintStream;

import java.util.Stack;

import org.aspectj.lang.*;
import java.io.*;
import java.util.*;
import java.text.*;
import java.lang.reflect.*;



abstract aspect TracerAspect {
	protected static int seq = 0;
	protected static boolean wastracing = false;
	protected static PrintStream out = System.out;
	protected static TracerFile tracefile = null;
	
	private static final int BEGIN = 0;
	private static final int END = 1;
	private static int current = 48;
	
	
	protected static void trace_record(JoinPoint jp, int mode) {
		String className = null;
		String methodName = null;
		String sourceLocation = null;
		String threadName = null;
		String instance = null;
		
		if(isTracing() == false) {
			return;
		}
							
		sourceLocation = jp.getSourceLocation().toString(); //retorna o arquivo e linha origem no c�igo fonte
			
		className = jp.getSignature().getDeclaringType().getName();
		methodName = jp.getSignature().getName();
		threadName = Thread.currentThread().getName();		
		int number = (jp.getTarget()!=null?jp.getTarget().hashCode():0);
		
		TracerXML.getInstance().setThread(threadName);
		
		if(number!=0) {
			instance = Integer.toHexString(number);
		} 
		
		if(mode==BEGIN) {
			TracerXML.getInstance().startMessage();
			//TracerXML.getInstance().setAttribute("source", (sourceLocation.split(":"))[0]);
			//TracerXML.getInstance().setAttribute("line", Long.parseLong((sourceLocation.split(":"))[1]));
			
			TracerXML.getInstance().setAttribute("class", className);
			TracerXML.getInstance().setAttribute("method", methodName);
			TracerXML.getInstance().setAttribute("instance", "@"+instance);
			TracerXML.getInstance().setAttribute("thread", threadName);
			TracerXML.getInstance().setAttribute("time", formatTime(System.currentTimeMillis()));
		} else {
			TracerXML.getInstance().endMessage();
		}
											
	}
	
		
	protected static boolean isTracing() {
		
		return "true".equals(System.getProperty("tracer.istracing"))
			? true
			: false;
	}

	protected static String getLogfilename() {
		String lfn = System.getProperty("tracer.logfilename");
		if ("".equals(lfn)) {
			lfn = null;
		}
		return lfn;
	}

	protected static String getLabel() {
		return System.getProperty("tracer.label");
	}

	static int id = 0;

	public static int getId() {
		id++;
		return id;
	}

	static private DateFormat formatter =
		DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG);

	
	
	private static String formatTime(long t) {
		return formatter.format(new Date(t));
	}

	protected static boolean isFiltered(
		String callerclass,
		String callermethod,
		String calleeclass,
		String calleemethod) {
		int nfilters = 0;
		try {
			nfilters =
				Integer.parseInt(System.getProperty("tracer.ndynamicfilters"));

		} catch (Exception e) {
			return false;
		}
		for (int i = 1; i < (nfilters + 1); i++) {
			String propname = "tracer.filter." + i;
			String filteron = System.getProperty(propname + ".filteron");
			String matches = System.getProperty(propname + ".matches");
			String value = System.getProperty(propname + ".value");
			String isvalid = System.getProperty(propname + ".isvalid");
			if ("true".equals(isvalid)) {
				String tocompare = null;
				if ("Caller Class".equals(filteron)) {
					tocompare = callerclass;
				} else if ("Caller Method".equals(filteron)) {
					tocompare = callermethod;
				} else if ("Callee Class".equals(filteron)) {
					tocompare = calleeclass;
				} else if ("Callee Method".equals(filteron)) {
					tocompare = calleemethod;
				} else {
					throw new IllegalStateException("Unknown Filteron Condition");
				}

				if ("true".equals(matches)) {
					if (tocompare.indexOf(value) != -1) {
						return true;
					}
				} else if ("false".equals(matches)) {
					if (tocompare.indexOf(value) == -1) {
						return true;
					}
				}
			}
		}
		return false;
	}

	/*
	 * Crosscut part
	 */

	/**
	 * Application classes - left unspecified.
	 */
	//Aline abstract pointcut myClass(Object obj);
	abstract pointcut myClass();

	public pointcut myConstructor() : 
		myClass() &&
		execution(*.new (..)); 
					
	public pointcut myMethod() :
		myClass() &&
		execution(* *.*(..));
	
	
	Object around() : myConstructor() {
		trace_record(thisJoinPoint, BEGIN);
		
		Object ret = proceed();
		
		trace_record(thisJoinPoint, END);
				
		return ret;
	}
	
	

	Object around() : myMethod() {
		trace_record(thisJoinPoint, BEGIN);
		
		Object ret = proceed();
		
		trace_record(thisJoinPoint, END);
				
		return ret;
	}
	
	
	protected void trace_test(JoinPoint jp, JoinPoint.StaticPart th, Object caller) {
		System.out.println(th.getSignature().getDeclaringType().getName() + " calling " + jp.getSignature().getDeclaringType().getName());
		System.out.println(th.getSignature().getName() + " calling " +jp.getSignature().getName() +" on " +jp.getTarget());
	}
		
}
